#pragma rtGlobals=1		// Use modern global access method.

constant K_TFmaxparams=3
strconstant K_CWTMotherWavelets="Morlet;Paul;DOG"


Structure TFDecompParameters
	//Universal structure to be used for input parameters to any function producing a spectral estimate
	
	wave dw,xw				//input waves for XY-pair data
	wave dw_int				//input wave for waveform data
	wave magnitude,phase		//output waves
	wave scale,period,frequency					//output scale coordinates
	wave coi					//cone of influence
	variable getParams		//set to 1 when calling to retrieve info for creation of controls
	
	variable Xmin,Xmax		//minimum and maximum x values to define a subrange of the data
	variable Fmin,Fmax		//minimum and maximum frequency of interest
	variable freqRes			//desired frequency resolution; if 0, default resolution (1/series length)
								//if <1/length, data will be padded to obtain desired resolution
	
	struct TFDecompSetVar SV			//structures holding set-variable parameters specific to a spectral estimate method
	
	struct TFDecompPopMenu PM		//structures holding popup menu parameters specific to a spectral estimate method
endStructure

Structure TFDecompSetVar
	//parameters to create a set variable control
	string name[K_TFmaxparams]				//control name
	variable value	[K_TFmaxparams]			//value
	variable low[K_TFmaxparams],high[K_TFmaxparams]	//low and high limits and increment for creating set variable control
	variable inc[K_TFmaxparams]	
endStructure

Structure TFDecompPopMenu
	//parameters to create a popup menu control
	string name[K_TFmaxparams]				//control name
	string value[K_TFmaxparams]				//semicolon-separated list for popup menu
	string popStr[K_TFmaxparams]				//value of selected item
	variable popNum[K_TFmaxparams]			//number of selected item
endStructure

Function TFDECOMPOSITION_prototype(TFPs)
	struct TFDecompParameters &TFPs
	
	if(TFPs.getParams)
		TFPs.SV.name[0]="";TFPs.SV.value[0]=nan;TFPs.SV.low[0]=nan;TFPs.SV.high[0]=nan;TFPs.SV.inc[0]=nan
		TFPs.SV.name[1]="";TFPs.SV.value[1]=nan;TFPs.SV.low[1]=nan;TFPs.SV.high[1]=nan;TFPs.SV.inc[1]=nan
		TFPs.SV.name[2]="";TFPs.SV.value[2]=nan;TFPs.SV.low[2]=nan;TFPs.SV.high[2]=nan;TFPs.SV.inc[2]=nan
		TFPs.PM.name[0]="";TFPs.PM.value[0]="";TFPs.PM.popStr[0]="";TFPs.PM.popNum[0]=nan
		TFPs.PM.name[1]="";TFPs.PM.value[1]="";TFPs.PM.popStr[1]="";TFPs.PM.popNum[1]=nan
		TFPs.PM.name[2]="";TFPs.PM.value[2]="";TFPs.PM.popStr[2]="";TFPs.PM.popNum[2]=nan
	else
//		TFPs.magnitude=nan;TFPs.phase=nan
	endif	
end

Function TFDECOMPOSITION_Wigner(TFPs)		//calculates Wigner transform
	struct TFDecompParameters &TFPs
	
	if(TFPs.getParams)
		TFPs.SV.name[0]="Gauss width fraction";TFPs.SV.value[0]=0.25;TFPs.SV.low[0]=0.05;TFPs.SV.high[0]=inf;TFPs.SV.inc[0]=0.05
		TFPs.SV.name[1]="";TFPs.SV.value[1]=nan;TFPs.SV.low[1]=nan;TFPs.SV.high[1]=nan;TFPs.SV.inc[1]=nan
		TFPs.SV.name[2]="";TFPs.SV.value[2]=nan;TFPs.SV.low[2]=nan;TFPs.SV.high[2]=nan;TFPs.SV.inc[2]=nan
		TFPs.PM.name[0]="";TFPs.PM.value[0]="";TFPs.PM.popStr[0]="";TFPs.PM.popNum[0]=nan
		TFPs.PM.name[1]="";TFPs.PM.value[1]="";TFPs.PM.popStr[1]="";TFPs.PM.popNum[1]=nan
		TFPs.PM.name[2]="";TFPs.PM.value[2]="";TFPs.PM.popStr[2]="";TFPs.PM.popNum[2]=nan
	else
		variable PtMax,dFreq,dx,gWidth,npts,newn,dwmean,fac
		wavestats/q/m=1 TFPs.dw_int
		dwmean=v_avg;npts=v_npnts
		dx=deltax(TFPs.dw_int)
		gWidth=TFPs.SV.value[0]*npts*dx
		
		TFPs.dw_int-=dwmean		//remove mean
		WignerTransform/DEST=$getwavesdatafolder(TFPs.magnitude,2)/GAUS=(gWidth) TFPs.dw_int
		TFPs.dw_int+=dwmean		//add mean back in
		
		dFreq=DimDelta(TFPs.magnitude,1)
		PtMax=floor(TFPs.Fmin/dFreq)
		DeletePoints/M=1 0,PtMax,TFPs.magnitude		//remove low frequency components to save memory
	
		fac=2/(min(2*(gWidth/dx),npts))	//Still doesn't deal with gauss widths < npts right
		MatrixOP/O TFPs.magnitude=sqrt(TFPs.magnitude)*fac				//adjust amplitudes
		
		copyscales TFPs.dw_int,TFPs.magnitude
		redimension/n=(dimsize(TFPs.magnitude,1)+1) TFPs.frequency,TFPs.period
		setscale/p y dFreq*PtMax,dFreq,TFPs.magnitude
		setscale/p x dFreq*PtMax,dFreq,TFPs.frequency,TFPs.period
		TFPs.frequency=x;TFPs.period=1/x
	endif
end

function TFDECOMPOSITION_Wavelet(TFPs)				//calculates continuous wavelet transform
	struct TFDecompParameters &TFPs
	
	if(TFPs.getParams)
		TFPs.SV.name[0]="Wavelet parameter"
		TFPs.SV.value[0]=8;TFPs.SV.low[0]=5;TFPs.SV.high[0]=30;TFPs.SV.inc[0]=1
		TFPs.SV.name[1]="Scale resolution"
		TFPs.SV.value[1]=0.05;TFPs.SV.low[1]=0;TFPs.SV.high[1]=1;TFPs.SV.inc[1]=0.02
		TFPs.SV.name[2]="";TFPs.SV.value[2]=nan;TFPs.SV.low[2]=nan;TFPs.SV.high[2]=nan;TFPs.SV.inc[2]=nan
		TFPs.PM.name[0]="Mother wavelet"
		TFPs.PM.value[0]=K_CWTMotherWavelets
		TFPs.PM.popStr[0]="Morlet"
		TFPs.PM.popNum[0]=whichlistitem(TFPs.PM.popStr[0],TFPs.PM.value[0])
		TFPs.PM.name[1]="";TFPs.PM.value[1]="";TFPs.PM.popStr[1]="";TFPs.PM.popNum[1]=nan
		TFPs.PM.name[2]="";TFPs.PM.value[2]="";TFPs.PM.popStr[2]="";TFPs.PM.popNum[2]=nan
	else
		variable dmean,dvar,dx,npts,npad,jlast,s0,pfac,magfac
		wavestats/q TFPs.dw_int
		dmean=v_avg;dvar=v_sdev^2;npts=v_npnts		//get stats
		dx=deltax(TFPs.dw_int)
		npad=2^(ceil(log(npts)/log(2))+1)
		
		StrSwitch(TFPs.PM.popStr[0])				//find wavelet mother-dependent factors to adjust scale to equivalent period
			case "Paul":
				pfac=4*pi/(2*TFPs.SV.value[0]+1)
				break
			case "DOG":
				pfac=2*pi/sqrt(TFPs.SV.value[0]+0.5)
				break
			case "Morlet":
	//		case kMotherMorletC:		complex Morlet decomposition not yet implemented
				pfac=4*pi/(TFPs.SV.value[0]+sqrt(2+TFPs.SV.value[0]^2))
				break
		endSwitch
		s0=(1/TFPs.Fmax)/dx/pfac
		jlast=floor((ln(npts)/ln(2))/TFPs.SV.value[1])	//find number of scales
		jlast=min(jlast,ceil((ln((1/TFPs.Fmin)/dx)/ln(2))/TFPs.SV.value[1]))
			
		redimension/n=(npad) TFPs.dw_int;TFPs.dw_int[npts,]=dmean		//pad data to minimize end effects
		setscale/p x 0,1,TFPs.dw_int											//dependence of CWT on input data x scaling is weird
		StrSwitch(TFPs.PM.popStr[0])				//parameters are different for different wavelet mothers, so we can't use a single generic call
			case "Morlet":
	//		case kMotherMorletC:		complex Morlet decomposition not yet implemented
				CWT/ENDM=2/OUT=4/Q/R2={s0,TFPs.SV.value[1],jlast}/SMP2=4/WBI1={Morlet}/WPR1={TFPs.SV.value[0]} TFPs.dw_int
				break
			case "Paul":
				CWT/ENDM=2/OUT=4/Q/R2={s0,TFPs.SV.value[1],jlast}/SMP2=4/WBI1={Paul,TFPs.SV.value[0]} TFPs.dw_int
				break
			case "DOG":
				CWT/ENDM=2/OUT=4/Q/R2={s0,TFPs.SV.value[1],jlast}/SMP2=4/WBI1={DOG,TFPs.SV.value[0]} TFPs.dw_int
				break
		endSwitch
		redimension/n=(npts) TFPs.dw_int
		setscale/p x TFPs.Xmin,dx,TFPs.dw_int
		
	//put waves in CWT folder	
		wave M_CWT,W_CWTScaling
		duplicate/o/R=[0,npts] M_CWT,TFPs.magnitude		//remove padding from wavelet transform
		duplicate/o W_CWTScaling,TFPs.scale,TFPs.period,TFPs.frequency
		killwaves M_CWT,W_CWTScaling
		redimension/n=(npts) TFPs.coi
		copyscales/p TFPs.dw_int,TFPs.magnitude,TFPs.coi
		
		magfac=2/npts/sqrt(npts)
		TFPs.magnitude[][]*=magfac/sqrt(TFPs.scale[q])									//adjust amplitude scaling
		TFPs.scale*=dx
		TFPs.period=TFPs.scale*pfac;TFPs.frequency=1/TFPs.period	//correct for original delta x and calculate Fourier equivalent period
	
		TFPs.coi[0,npts/2-1]=dx*pfac*(sqrt(2))^((!StringMatch(TFPs.PM.popStr[0],"Paul")) ? 1 : -1)*1*p		//get cone of influence
		TFPs.coi[npts/2,npts-1]=TFPs.coi[npts-1-p]
		TFPs.coi[]=binarysearchinterp(scale,TFPs.coi[p])	//convert COI values from scales to points
	endif
end

