#pragma rtGlobals=1		// Use modern global access method.
#pragma version=0.1
#pragma IgorVersion=6.02A

//#include "Interpolations"
//#include "Spectral Estimates"
//#include "AR(1)"
//#include "SSA control"
//#include "Age_Depth"
//#include "Time-Freq decompositions"
//#include "Lomb periodogram"
//#include "Control utilities"

static constant kMaxNameLength=33		//Igor names are restricted to 31 characters; add 2 for enclosing single-quotes in liberal names
static constant kTFVersion=010			//current Time-Frequency procedure version
static constant kTFCSVersion=010		//last change in TF_ControlSettings structure - loading structures from previous versions will break

static constant kDataSelectionTab=0
static constant kInterpolationTab=1
static constant kTFDecompTab=2
static constant kSpectrumTab=3
static constant kAppearanceTab=4
static constant kTuningTab=5

strconstant KPopupvalsVertAxis="Linear frequency;Linear period;Log(10);Log(2)"
strconstant KPopupvalsAmpAxis="Linear;Log(10);Log(2)"
strconstant KPopupvalsNorms="Magnitude;AR normalized"

Menu "Analysis"
	"Time-Frequency Investigation",/Q
end

Static Structure TF_GlobalRefs							//structure containing references to all globals, so they don't have to be redefined constantly
	
//waves
//data
	wave dw,xw,dw_int								//wave references for user selected data and x-value waves and for interpolated data
//decomposition
	wave TFmag,TFfrequency,TFperiod,TFscale,TFcoi,TFBGnorm,TFGlobalSpec
//spectrum
	wave AR1Mag,AR1Frequency,AR1Period				//wave references for expected AR1 red noise spectrum
	wave PgramMag,PgramPh,PgramFrequency,PgramPeriod	//wave references for periodogram
//appearance
	wave CTabMod									//wave reference for color table to display time-frequency decomposition
	wave/t vertP,vertF
	wave vertPy,vertFy								//manual tick waves
//tuning
	wave xw_tuned,IntX_tuned						//wave references for tuned x-value waves
	wave origTie,tunedTie							//wave references for tuning ties
	wave frequencyTie,periodTie		//wave references for tuning ties
//for display
	wave Plotxw
	wave PlotTFdecomp,PlotXTFdecomp
	wave PlotXAR1,PlotXPgram
	wave PlotXTie

//global variables
	SVAR win								//name of window
//data selection tab
	SVAR dwPath,xwPath
	NVAR RemoveOutliers,RemoveFliers
	NVAR RemoveSD,RemoveRG						//standard deviation and data window to define fliers/outliers
//interpolation tab
	SVAR InterpType									//current setting of TF interpolation popup selector
	NVAR LiveInterp									//auto-interpolate setting
	NVAR InterpRes,Xmin,Xmax						//interpolation parameters
	NVAR InterpSV0,InterpSV1,InterpSV2			//current setting of interpolation setvariable parameters
	SVAR InterpPM0,InterpPM1,InterpPM2		//current setting of interpolation popupmenu parameters
//decomposition tab
	SVAR DecompType									//current setting of TF decomposition popup selector
	NVAR PeriodMin,PeriodMax						//decomposition parameters
	NVAR TFDecompSV0,TFDecompSV1,TFDecompSV2	//current setting of decomposition setvariable parameters
	SVAR TFDecompPM0,TFDecompPM1,TFDecompPM2	//current setting of decomposition popupmenu parameters
//spectrum tab
	SVAR SpectrumType								//current setting of TF spectrum popup selector
	NVAR SpectrumSV0,SpectrumSV1,SpectrumSV2	//current setting of spectrum setvariable parameters
	SVAR SpectrumPM0,SpectrumPM1,SpectrumPM2	//current setting of spectrum popupmenu parameters
	NVAR SigLevel
//appearance tab
	SVAR AmpAxis
	SVAR Norms										//current setting of normalization popup selector
	SVAR VertAxis										//current setting of vertical axis popup selector
	SVAR CTabMode									//color table
	NVAR CTabMax,CTabMin,CTabTau				//color table parameters
	NVAR PTickSpace,FTickSpace,labelSpace			//axis tick parameters
//tuning tab
	NVAR TuningPeriod,TuningRegX,TuningRegX1	//tuning parameters
	NVAR ShowTuned									//0 for original data, non-0 for tuned data
	
//Folders
	string TFfolder
	string SettingsFolder
	string AR1Folder
	string SpectrumFolder
	string TFDecompFolder
	string TuningFolder
endStructure

function TF_MakeRefs(TFgr,win)			//instead of defining lists of references in each function, a single call to this takes care of all
	struct TF_GlobalRefs &TFgr
	string win
		
	TFgr.TFfolder=getuserdata(win,"","TFfolder")
	if(datafolderexists(TFgr.TFfolder))
	//Folders
		TFgr.SettingsFolder=TFgr.TFfolder+":Settings";NewDataFolderPath(TFgr.SettingsFolder)
	
		string/g $(TFgr.SettingsFolder+":win")
		SVAR TFgr.win=$(TFgr.SettingsFolder+":win")
	//global variables
		string/g $(TFgr.SettingsFolder+":dwPath"),$(TFgr.SettingsFolder+":xwPath")
		SVAR TFgr.dwPath=$(TFgr.SettingsFolder+":dwPath"), TFgr.xwPath=$(TFgr.SettingsFolder+":xwPath")
	//data selection tab
		string/g $(TFgr.SettingsFolder+":dwPath"),$(TFgr.SettingsFolder+":xwPath")
		SVAR TFgr.dwPath=$(TFgr.SettingsFolder+":dwPath"),TFgr.xwPath=$(TFgr.SettingsFolder+":xwPath")
		variable/g $(TFgr.SettingsFolder+":RemoveOutliers"),$(TFgr.SettingsFolder+":RemoveFliers")
		NVAR TFgr.RemoveOutliers=$(TFgr.SettingsFolder+":RemoveOutliers"),TFgr.RemoveFliers=$(TFgr.SettingsFolder+":RemoveFliers")
		variable/g $(TFgr.SettingsFolder+":RemoveSD"),$(TFgr.SettingsFolder+":RemoveRG")
		NVAR TFgr.RemoveSD=$(TFgr.SettingsFolder+":RemoveSD"),TFgr.RemoveRG=$(TFgr.SettingsFolder+":RemoveRG")
	//interpolation tab
		string/g $(TFgr.SettingsFolder+":InterpType")
		SVAR TFgr.InterpType=$(TFgr.SettingsFolder+":InterpType")
		variable/g $(TFgr.SettingsFolder+":InterpRes"),$(TFgr.SettingsFolder+":LiveInterp")
		NVAR TFgr.InterpRes=$(TFgr.SettingsFolder+":InterpRes"),TFgr.LiveInterp=$(TFgr.SettingsFolder+":LiveInterp")
		variable/g $(TFgr.SettingsFolder+":Xmin"),$(TFgr.SettingsFolder+":Xmax")
		NVAR TFgr.Xmin=$(TFgr.SettingsFolder+":Xmin"),TFgr.Xmax=$(TFgr.SettingsFolder+":Xmax")
		NVAR/Z TFgr.InterpSV0=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV0")
		NVAR/Z TFgr.InterpSV1=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV1")
		NVAR/Z TFgr.InterpSV2=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":SV2")
		SVAR/Z TFgr.InterpPM0=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM0")
		SVAR/Z TFgr.InterpPM1=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM1")
		SVAR/Z TFgr.InterpPM2=$(TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType+":PM2")
	//decomposition tab
		string/g $(TFgr.SettingsFolder+":DecompType")
		SVAR TFgr.DecompType=$(TFgr.SettingsFolder+":DecompType")
		variable/g $(TFgr.SettingsFolder+":PeriodMin"),$(TFgr.SettingsFolder+":PeriodMax")
		NVAR TFgr.PeriodMin=$(TFgr.SettingsFolder+":PeriodMin"),TFgr.PeriodMax=$(TFgr.SettingsFolder+":PeriodMax")
		variable/g $(TFgr.SettingsFolder+":WignerWidth"),$(TFgr.SettingsFolder+":WignerGauss")
		NVAR/Z TFgr.TFDecompSV0=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV0")
		NVAR/Z TFgr.TFDecompSV1=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV1")
		NVAR/Z TFgr.TFDecompSV2=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":SV2")
		SVAR/Z TFgr.TFDecompPM0=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM0")
		SVAR/Z TFgr.TFDecompPM1=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM1")
		SVAR/Z TFgr.TFDecompPM2=$(TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType+":PM2")
	//spectrum tab
		string/g $(TFgr.SettingsFolder+":SpectrumType")
		SVAR TFgr.SpectrumType=$(TFgr.SettingsFolder+":SpectrumType")
		NVAR/Z TFgr.SpectrumSV0=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV0")
		NVAR/Z TFgr.SpectrumSV1=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV1")
		NVAR/Z TFgr.SpectrumSV2=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":SV2")
		SVAR/Z TFgr.SpectrumPM0=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM0")
		SVAR/Z TFgr.SpectrumPM1=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM1")
		SVAR/Z TFgr.SpectrumPM2=$(TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType+":PM2")
		variable/g $(TFgr.SettingsFolder+":SigLevel")
		NVAR TFgr.SigLevel=$(TFgr.SettingsFolder+":SigLevel")
	//appearance tab
		string/g $(TFgr.SettingsFolder+":AmpAxis"),$(TFgr.SettingsFolder+":VertAxis")
		SVAR TFgr.AmpAxis=$(TFgr.SettingsFolder+":AmpAxis"),TFgr.VertAxis=$(TFgr.SettingsFolder+":VertAxis")
		string/g $(TFgr.SettingsFolder+":Norms")
		SVAR TFgr.Norms=$(TFgr.SettingsFolder+":Norms")
		string/g $(TFgr.SettingsFolder+":CTabMode")
		SVAR TFgr.CTabMode=$(TFgr.SettingsFolder+":CTabMode")
		if(strlen(TFgr.CTabMode)==0)
			TFgr.CTabMode="Terrain"
		endif
		variable/g $(TFgr.SettingsFolder+":CTabMax"),$(TFgr.SettingsFolder+":CTabMin")
		NVAR TFgr.CTabMax=$(TFgr.SettingsFolder+":CTabMax"),TFgr.CTabMin=$(TFgr.SettingsFolder+":CTabMin")
		variable/g $(TFgr.SettingsFolder+":CTabTau")
		NVAR TFgr.CTabTau=$(TFgr.SettingsFolder+":CTabTau")
		variable/g $(TFgr.SettingsFolder+":PTickSpace"),$(TFgr.SettingsFolder+":FTickSpace")
		NVAR TFgr.PTickSpace=$(TFgr.SettingsFolder+":PTickSpace"),TFgr.FTickSpace=$(TFgr.SettingsFolder+":FTickSpace")
		variable/g $(TFgr.SettingsFolder+":LabelSpace")
		NVAR TFgr.LabelSpace=$(TFgr.SettingsFolder+":LabelSpace")
	//tuning tab
		variable/g $(TFgr.SettingsFolder+":TuningPeriod")
		NVAR TFgr.TuningPeriod=$(TFgr.SettingsFolder+":TuningPeriod")
		variable/g $(TFgr.SettingsFolder+":TuningRegX"),$(TFgr.SettingsFolder+":TuningRegX1")
		NVAR TFgr.TuningRegX=$(TFgr.SettingsFolder+":TuningRegX"),TFgr.TuningRegX1=$(TFgr.SettingsFolder+":TuningRegX1")
		variable/g $(TFgr.SettingsFolder+":ShowTuned")
		NVAR TFgr.ShowTuned=$(TFgr.SettingsFolder+":ShowTuned")

//Folders
		TFgr.AR1Folder=TFgr.TFfolder+":AR1";NewDataFolderPath(TFgr.AR1Folder)
		TFgr.SpectrumFolder=TFgr.TFfolder+":Spectrum:"+TFgr.SpectrumType;NewDataFolderPath(TFgr.SpectrumFolder)
		TFgr.TFDecompFolder=TFgr.TFfolder+":TFDecomposition:"+TFgr.DecompType
		if(!StringMatch(TFgr.DecompType,"none"))
			NewDataFolderPath(TFgr.TFDecompFolder)
		endif
		TFgr.TuningFolder=TFgr.TFfolder+":Tuning";NewDataFolderPath(TFgr.TuningFolder)
	
	//waves
	//data
		wave/Z TFgr.dw=$(TFgr.dwPath),TFgr.xw=$(TFgr.xwPath)
		wave/Z TFgr.dw_int=$(TFgr.TFfolder+":IntData")
	//decomposition
		wave/Z TFgr.TFmag=$(TFgr.TFDecompFolder+":magnitude"),TFgr.TFGlobalSpec=$(TFgr.TFDecompFolder+":GlobalSpec"),TFgr.TFBGnorm=$(TFgr.TFDecompFolder+":BGnorm")
		wave/Z TFgr.TFperiod=$(TFgr.TFDecompFolder+":period"),TFgr.TFfrequency=$(TFgr.TFDecompFolder+":frequency"),TFgr.TFscale=$(TFgr.TFDecompFolder+":scale")
		wave/Z TFgr.TFcoi=$(TFgr.TFDecompFolder+":COI")
	//spectrum
		wave/Z TFgr.AR1Mag=$(TFgr.AR1folder+":Amplitude"),TFgr.AR1period=$(TFgr.AR1folder+":period"),TFgr.AR1frequency=$(TFgr.AR1folder+":frequency")
		wave/Z TFgr.PgramMag=$(TFgr.SpectrumFolder+":Amplitude"),TFgr.PgramPh=$(TFgr.SpectrumFolder+":Phase")
		wave/Z TFgr.PgramFrequency=$(TFgr.SpectrumFolder+":Frequency"),TFgr.PgramPeriod=$(TFgr.SpectrumFolder+":Period")
	//appearance
		wave/Z TFgr.CTabMod=$(TFgr.TFFolder+":CTabMod")
		wave/Z/t TFgr.vertP=$(TFgr.TFfolder+":vertP"),TFgr.vertF=$(TFgr.TFfolder+":vertF")
		wave/Z TFgr.vertPy=$(TFgr.TFfolder+":vertPy"),TFgr.vertFy=$(TFgr.TFfolder+":vertFy")			//manual tick waves for periods
	//tuning
		wave/Z TFgr.IntX_tuned=$(TFgr.TFfolder+":IntDataX_tuned")
		string xwPrefix
		if(waveexists(TFgr.xw))
			xwPrefix=getwavesdatafolder(TFgr.xw,2)
		elseif(waveexists(TFgr.dw))
			xwPrefix=getwavesdatafolder(TFgr.dw,2)+"X"
		else
			xwPrefix=""
		endif
		wave/Z TFgr.xw_tuned=$(xwPrefix+"_tuned")
		wave/Z TFgr.origTie=$(xwPrefix+"_origTie"),TFgr.tunedTie=$(xwPrefix+"_tunedTie")
		wave/Z TFgr.frequencyTie=$(TFgr.TuningFolder+":frequencyTie"),TFgr.periodTie=$(TFgr.TuningFolder+":periodTie")
	
	//for display
		wave/Z TFgr.Plotxw=$(" ")
		wave/Z TFgr.PlotTFdecomp=$(" "),TFgr.PlotXTFdecomp=$(" ")
		wave/Z TFgr.PlotXAR1=$(" "),TFgr.PlotXPgram=$(" ")
		wave/Z TFgr.PlotXTie=$(" ")
		if(TFgr.ShowTuned && WaveExists(TFgr.xw_tuned))						//if showing tuned data
			wave/Z TFgr.Plotxw=$getWavesDataFolder(TFgr.xw_tuned,2)
		elseif(WaveExists(TFgr.xw))
			wave/Z TFgr.Plotxw=$getWavesDataFolder(TFgr.xw,2)
		endif
		strSwitch(TFgr.Norms)
			case "Magnitude":
				if(waveexists(TFgr.TFmag))
					wave TFgr.PlotTFdecomp=$getWavesDataFolder(TFgr.TFmag,2)
				endif
				break
			case "AR normalized":
				if(waveexists(TFgr.TFBGnorm))
					wave TFgr.PlotTFdecomp=$getWavesDataFolder(TFgr.TFBGnorm,2)
				endif
				break
		endSwitch
		strswitch(TFgr.VertAxis)
			case "Linear frequency":
				if(WaveExists(TFgr.AR1Frequency))
					wave TFgr.PlotXAR1=$getWavesDataFolder(TFgr.AR1Frequency,2)
				endif
				if(WaveExists(TFgr.PgramFrequency))
					wave TFgr.PlotXPgram=$getWavesDataFolder(TFgr.PgramFrequency,2)
				endif
				if(WaveExists(TFgr.frequencyTie))
					wave TFgr.PlotXTie=$getWavesDataFolder(TFgr.frequencyTie,2)
				endif
				if(WaveExists(TFgr.TFfrequency))
					wave TFgr.plotXTFdecomp=$getWavesDataFolder(TFgr.TFfrequency,2)
				endif
				break
			case "Linear period":
			case "Log(2)":
			case "Log(10)":
				if(WaveExists(TFgr.AR1Period))
					wave TFgr.PlotXAR1=$getWavesDataFolder(TFgr.AR1Period,2)
				endif
				if(WaveExists(TFgr.PgramPeriod))
					wave TFgr.PlotXPgram=$getWavesDataFolder(TFgr.PgramPeriod,2)
				endif
				if(WaveExists(TFgr.periodTie))
					wave TFgr.PlotXTie=$getWavesDataFolder(TFgr.periodTie,2)
				endif
				if(WaveExists(TFgr.TFperiod))
					wave TFgr.plotXTFdecomp=$getWavesDataFolder(TFgr.TFperiod,2)
				endif
				break
		endSwitch
	endif
end

function TF_LoadSettings(TFgr,dwPath,win)
	struct TF_GlobalRefs &TFgr
	string dwPath,win
	
	SVAR/Z newwin=$(dwPath+"_TF:settings:win")
	wave dw=$dwPath
	if(waveexists(dw))					//Check whether data wave is being changed
		if(SVAR_exists(newwin))		//has wave been analyzed?
			if(strlen(newwin)>0 && cmpstr(newwin,win))
				DoWindow/F $(newwin)
			else
				v_flag=0
			endif
		else
			V_flag=0
		endif
		if(V_flag)							//wave analyzed in already open window
			WBL_SetSelectedWave("DataWaveSelect_tab"+num2istr(kDataSelectionTab),win,TFgr.dw)	//reset data wave selector to previous selection
			doalert 0,"Wave is analyzed in a different window!"
			return 0											//proper window has already been brought to front, so should exit
		else								//need to change the settings for this window
			SetWindow $win userdata(TFfolder)=(dwPath+"_TF")	//redirect window user data TFfolder to newly selected dw
			TFgr.win=""					//detach previous dw selection from this window
			TF_MakeRefs(TFgr,win)	
			if(!datafolderexists(TFgr.TFfolder))		//never before analyzed
				newdatafolderpath(dwPath+"_TF")
				TF_MakeRefs(TFgr,win)
			//Control states
				TFgr.dwPath=dwPath
				TF_SetParameterDefaults(TFgr,1)
			endif
			TFgr.win=win		//attach newly selected dw to this window
			TF_BuildControlPanel(TFgr)

			//Control states
			if(waveexists(TFgr.dw))
				WBL_SetSelectedWave("DataWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,TFgr.dw)
			endif
			if(waveexists(TFgr.xw))
				WBL_SetSelectedWave("TimeWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,TFgr.xw)
			else
				WBL_SetSelectedFolder("TimeWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,getwavesdatafolder(TFgr.dw,1))
			endif
			struct WMPopupAction PUs

			TF_SpectrumOptions(TFgr)
			TF_DecompOptions(TFgr)
		endif
	endif
//set to current tab
	struct WMTabControlAction TCs
	controlInfo/W=$TFgr.win Tabs
	TCs.tab=V_value
	TCs.win=TFgr.win
	TCs.eventcode=2
	TFTabs(TCs)
end

Function TimeFrequencyInvestigation()		//builds initial controls
	
	string win=uniquename("TimeFrequency",6,0)
//Build window
	Display/K=1/W=(5,44,705,764)/N=$(win) as "Time Frequency Plot"
	Modifygraph margin(left)=72,margin(bottom)=72,margin(right)=(72)
	ControlBar 120
	Button DisplayHelp pos={615,95},size={75,20},proc=TFButton,title="Help"
//Build controls for data selection - other controls will appear only when data wave has been selected
	TabControl Tabs,pos={0,0},size={700,120},proc=TFTabs,tabLabel(0)="Data selection"
	WaveBrowserListbox("DataWaveSelect_tab"+num2istr(kDataSelectionTab),win,promptText="Select data wave:",Xpos=5,Ypos=20,Xsize=150,Ysize=95,WaveOpts="CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0",procName="TFWBListbox")
	SetWindow $win userdata(TFfolder)=(" ")		//because of bug: datafolderexists("") returns 1
end	

Function/s TFListFunctions(prefix,none)
	string prefix
	variable none
	
	string FuncList=FunctionList(prefix+"_*",";","NPARAMS:1,VALTYPE:1,KIND:6")
	FuncList=RemoveFromList(prefix+"_prototype",FuncList)
	FuncList=ReplaceString(prefix+"_",FuncList,"")
	
	if(none)
		FuncList="none;"+FuncList
	endif	
	return FuncList
end

Function TF_SetParameterDefaults(TFgr,new)
	struct TF_GlobalRefs &TFgr
	variable new
	
	TF_MakeRefs(TFgr,TFgr.win)
	
	if(new)
	//global variables
	//data selection tab
		TFgr.RemoveSD=3;TFgr.RemoveRG=1							//statistically "bad" parameters
	//interpolation tab
		TFgr.InterpType=stringfromlist(0,TFListFunctions("INTERPOLATION",0))
		TFgr.InterpRes=nan;TFgr.Xmin=nan;TFgr.Xmax=nan			//interpolation parameters
		TFgr.LiveInterp=1
	//decomposition tab
		TFgr.DecompType=stringfromlist(0,TFListFunctions("TFDECOMPOSITION",1))
		TFgr.PeriodMin=nan;TFgr.PeriodMax=nan						//decomposition parameters
	//spectrum tab
		TFgr.SpectrumType=stringfromlist(0,TFListFunctions("SPECTRUM",1))
		TFgr.sigLevel=95													//significance level for spectrum display
	//appearance tab
		TFgr.VertAxis=stringfromlist(2,KPopupvalsVertAxis)
		TFgr.AmpAxis=stringfromlist(0,KPopupvalsAmpAxis)
		TFgr.Norms=stringfromlist(0,KPopupvalsNorms)
		TFgr.CTabMax=nan;TFgr.CTabMin=nan;TFgr.CTabTau=10		//color table parameters
		TFgr.PTickSpace=1;TFgr.FTickSpace=1;TFgr.LabelSpace=5	//axis parameters
		TFgr.CTabMode="Terrain"			
	//tuning tab
		TFgr.TuningPeriod=1												//tuning parameters
		TFgr.TuningRegX=nan;TFgr.TuningRegX1=nan					//tuning parameters
	endif

	variable np=numpnts(TFgr.dw)
	if(TFgr.ShowTuned)
		differentiate/P/EP=1/METH=1 TFgr.xw_tuned /D=$(TFgr.SettingsFolder+":tmp_dxw")
		wave tmp_dxw=$(TFgr.SettingsFolder+":tmp_dxw")
		sort tmp_dxw,tmp_dxw
		TFgr.InterpRes=tmp_dxw[(np-2)/2]		//calculate median delta
		wavestats/M=1/q TFgr.xw_tuned
		TFgr.Xmin=v_min+TFgr.InterpRes/2;TFgr.Xmax=v_max-TFgr.InterpRes/2
		killwaves tmp_dxw
	elseif(waveexists(TFgr.xw))
		differentiate/P/EP=1/METH=1 TFgr.xw /D=$(TFgr.SettingsFolder+":tmp_dxw")
		wave tmp_dxw=$(TFgr.SettingsFolder+":tmp_dxw")
		sort tmp_dxw,tmp_dxw
		TFgr.InterpRes=tmp_dxw[(np-2)/2]		//calculate median delta
		wavestats/M=1/q TFgr.xw
		TFgr.Xmin=v_min+TFgr.InterpRes/2;TFgr.Xmax=v_max-TFgr.InterpRes/2
		killwaves tmp_dxw
	elseif(waveexists(TFgr.dw))
		TFgr.InterpRes=deltax(TFgr.dw)			//use data deltax
		if(TFgr.InterpRes>0)
			TFgr.Xmin=pnt2x(TFgr.dw,0);TFgr.Xmax=pnt2x(TFgr.dw,numpnts(TFgr.dw)-1)
		else
			TFgr.Xmax=pnt2x(TFgr.dw,0);TFgr.Xmin=pnt2x(TFgr.dw,numpnts(TFgr.dw)-1)
		endif
	endif
	TFgr.InterpRes=abs(TFgr.InterpRes)
	TFgr.PeriodMin=2*TFgr.InterpRes;TFgr.PeriodMax=(TFgr.Xmax-TFgr.Xmin)/4

	TF_BuildControlPanel(TFgr)
	WBL_SetSelectedWave("DataWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,TFgr.dw)
	if(waveexists(TFgr.xw))
		WBL_SetSelectedWave("TimeWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,TFgr.xw)
	else
		WBL_SetSelectedFolder("TimeWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,getwavesdatafolder(TFgr.dw,1))
	endif
end

Function TF_BuildControlPanel(TFgr)
	struct TF_GlobalRefs &TFgr
	
	variable np=numpnts(TFgr.dw)
	
	string WaveOpts
//***Tabs
	TabControl Tabs,tablabel(kInterpolationTab)="Interpolation",tablabel(kTFdecompTab)="TF decomp",win=$TFgr.win
	TabControl Tabs,tablabel(kSpectrumTab)="Spectrum",tablabel(kAppearanceTab)="Appearance",tablabel(kTuningTab)="Tuning",win=$TFgr.win
//***Data selection controls
	sprintf WaveOpts,"CMPLX:0,TEXT:0,INTEGER:0,DIMS:1,UNSIGNED:0,WORD:0,MAXROWS:%u,MINROWS:%u",np,np
	WaveBrowserListbox("TimeWaveSelect_tab"+num2istr(kDataSelectionTab),TFgr.win,promptText="For XY data, select Y wave:",Xpos=160,Ypos=20,Xsize=150,Ysize=95,WaveOpts=WaveOpts,procName="TFWBListbox")
//	CheckBox $("RemoveFliers_tab"+num2istr(kDataSelectionTab)) pos={325,45},size={100,20},proc=TFCheckBox,title="Remove fliers",variable=TFgr.RemoveFliers,win=$TFgr.win
//	CheckBox $("RemoveOutliers_tab"+num2istr(kDataSelectionTab)) pos={325,70},size={100,20},proc=TFCheckBox,title="Remove outliers",variable=TFgr.RemoveOutliers,win=$TFgr.win
//	Button $("SSA_tab"+num2istr(kDataSelectionTab)) pos={325,95},size={120,20},proc=TFButton,title="SSA enhancement",win=$TFgr.win
//***Interpolation controls
	PopupMenu $("InterpType_tab"+num2istr(kInterpolationTab)) pos={5,20},proc=TFPopup,title="Interpolation:",value=TFListFunctions("INTERPOLATION",0),mode=(max(0,WhichListItem(TFgr.InterpType,TFListFunctions("INTERPOLATION",0)))+1),win=$TFgr.win
	SetVariable $("InterpRes_tab"+num2istr(kInterpolationTab)) pos={5,45},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Interpolated resolution:",value=$(TFgr.SettingsFolder+":InterpRes"),win=$TFgr.win
	SetVariable $("XMin_tab"+num2istr(kInterpolationTab)) pos={5,70},size={150,20},bodywidth=70,limits={-inf,inf,1},proc=TFSetVariable,title="Minimum X value:",value=$(TFgr.SettingsFolder+":XMin"),win=$TFgr.win
	SetVariable $("XMax_tab"+num2istr(kInterpolationTab)) pos={5,95},size={150,20},bodywidth=70,limits={-inf,inf,1},proc=TFSetVariable,title="Maximum X value:",value=$(TFgr.SettingsFolder+":XMax"),win=$TFgr.win
	CheckBox $("LiveInterp_tab"+num2istr(kInterpolationTab)) pos={600,20},size={100,20},proc=TFCheckBox,title="Live update",variable=TFgr.LiveInterp,win=$TFgr.win
	Button $("ResetValues_tab"+num2istr(kInterpolationTab)) pos={600,45},size={100,20},proc=TFButton,title="Reset values",win=$TFgr.win
	Button $("DoInterpolate_tab"+num2istr(kInterpolationTab)) pos={600,70},size={100,20},proc=TFButton,title="Re-interpolate",win=$TFgr.win
//***TF decomposition controls
	PopupMenu $("DecompType_tab"+num2istr(kTFDecompTab)),pos={5,20},proc=TFPopup,title="Decomposition:",value=TFListFunctions("TFDECOMPOSITION",1),mode=(max(0,WhichListItem(TFgr.DecompType,TFListFunctions("TFDECOMPOSITION",1)))+1),win=$TFgr.win
//	Button $("Decompose_tab"+num2istr(kTFDecompTab)) pos={500,45},size={75,20},proc=TFButton,title="Calculate",win=$TFgr.win
	SetVariable $("PeriodMin_tab"+num2istr(kTFDecompTab)) pos={5,45},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Minimum period:",value=$(TFgr.SettingsFolder+":PeriodMin"),win=$TFgr.win
	SetVariable $("PeriodMax_tab"+num2istr(kTFDecompTab)) pos={5,70},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Maximum period:",value=$(TFgr.SettingsFolder+":PeriodMax"),win=$TFgr.win
//***Spectrum controls
	PopupMenu $("SpectrumType_tab"+num2istr(kSpectrumTab)),pos={5,20},proc=TFPopup,title="Spectrum",value=TFListFunctions("SPECTRUM",1),mode=(max(0,WhichListItem(TFgr.SpectrumType,TFListFunctions("SPECTRUM",1)))+1),win=$TFgr.win
	SetVariable $("PeriodMin_tab"+num2istr(kSpectrumTab)) pos={5,45},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Minimum period:",value=$(TFgr.SettingsFolder+":PeriodMin"),win=$TFgr.win
	SetVariable $("PeriodMax_tab"+num2istr(kSpectrumTab)) pos={5,70},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Maximum period:",value=$(TFgr.SettingsFolder+":PeriodMax"),win=$TFgr.win
	SetVariable $("SigLevel_tab"+num2istr(kSpectrumTab)) pos={5,95},size={150,20},bodywidth=70,limits={0,inf,1},proc=TFSetVariable,title="Significance level:",value=$(TFgr.SettingsFolder+":SigLevel"),win=$TFgr.win	
//***Appearance controls
	PopupMenu $("VertAxis_tab"+num2istr(kAppearanceTab)),pos={5,20},mode=3,proc=TFPopup,title="Vertical axis:",value=KPopupvalsVertAxis,mode=(max(0,WhichListItem(TFgr.VertAxis,KPopupvalsVertAxis))+1),win=$TFgr.win
	PopupMenu $("AmpAxis_tab"+num2istr(kAppearanceTab)),pos={5,45},proc=TFPopup,title="Amplitude axis:",value=KPopupvalsAmpAxis,mode=(max(0,WhichListItem(TFgr.AmpAxis,KPopupvalsAmpAxis))+1),win=$TFgr.win
	PopupMenu $("Norms_tab"+num2istr(kAppearanceTab)),pos={5,70},proc=TFPopup,title="Normalization:",value=KPopupvalsNorms,mode=(max(0,WhichListItem(TFgr.Norms,KPopupvalsNorms))+1),win=$TFgr.win
	PopupMenu $("CTabMode_tab"+num2istr(kAppearanceTab)),pos={200,20},size={200,20},proc=TFPopup,title="ColorTable",value="*COLORTABLEPOP*",mode=(whichlistitem(TFgr.CTabMode,CTabList())+1),win=$TFgr.win
	Button $("CopyPlot_tab"+num2istr(kAppearanceTab)) pos={5,95},size={75,20},proc=TFButton,title="Make a copy",win=$TFgr.win
	SetVariable $("CTabTau_tab"+num2istr(kAppearanceTab)) pos={250,45},size={200,20},bodywidth=60,limits={1,inf,1},proc=TFSetVariable,title="Color table nonlinearity:",value=$(TFgr.SettingsFolder+":CTabTau"),win=$TFgr.win
	SetVariable $("CTabMin_tab"+num2istr(kAppearanceTab)) pos={250,70},size={200,20},bodywidth=70,limits={0,inf,0.1},proc=TFSetVariable,title="Minimum color at:",format="%3.3G",value=$(TFgr.SettingsFolder+":CTabMin"),win=$TFgr.win
	SetVariable $("CTabMax_tab"+num2istr(kAppearanceTab)) pos={250,95},size={200,20},bodywidth=70,limits={0,inf,0.1},proc=TFSetVariable,title="Maximum color at:",format="%3.3G",value=$(TFgr.SettingsFolder+":CTabMax"),win=$TFgr.win
	SetVariable $("PTickSpace_tab"+num2istr(kAppearanceTab)) pos={450,20},size={200,20},bodywidth=60,limits={1e-45,1e38,0.1},proc=TFSetVariable,title="spacing of Period ticks:",value=$(TFgr.SettingsFolder+":PTickSpace"),win=$TFgr.win
	SetVariable $("FTickSpace_tab"+num2istr(kAppearanceTab)) pos={450,45},size={200,20},bodywidth=60,limits={1e-45,1e38,0.1},proc=TFSetVariable,title="spacing of Frequency ticks:",value=$(TFgr.SettingsFolder+":FTickSpace"),win=$TFgr.win
	SetVariable $("LabelSpace_tab"+num2istr(kAppearanceTab)) pos={450,70},size={200,20},bodywidth=60,limits={0,inf,1},proc=TFSetVariable,title="label every nth tick:",value=$(TFgr.SettingsFolder+":LabelSpace"),win=$TFgr.win
//***Tuning controls
	Button $("TuneStep_tab"+num2istr(kTuningTab)) pos={5,20},size={75,20},proc=TFButton,title="Tune Step",win=$TFgr.win
	Button $("EditPicks_tab"+num2istr(kTuningTab)) pos={5,45},size={120,20},proc=TFButton,title="Edit tuning ties",win=$TFgr.win
	Button $("ResetPicks_tab"+num2istr(kTuningTab)) pos={5,70},size={120,20},proc=TFButton,title="Reset tuning ties",win=$TFgr.win
	Button $("DisplayCurve_tab"+num2istr(kTuningTab)) pos={200,20},size={150,20},proc=TFButton,title="Display transfer curve",win=$TFgr.win
	CheckBox $("ShowTuned_tab"+num2istr(kTuningTab)) pos={450,20},size={100,20},proc=TFCheckBox,title="Show tuned series",variable=TFgr.ShowTuned,win=$TFgr.win
	SetVariable $("TuningPeriod_tab"+num2istr(kTuningTab)) pos={200,70},size={150,20},bodywidth=60,limits={0,inf,1},proc=TFSetVariable,title="Period of tuning component:",value=$(TFgr.SettingsFolder+":TuningPeriod"),win=$TFgr.win
	SetVariable $("TuningRegX_tab"+num2istr(kTuningTab)) pos={200,90},size={150,20},bodywidth=60,limits={-inf,inf,1},proc=TFSetVariable,title="Registry Old X:",value=$(TFgr.SettingsFolder+":TuningRegX"),win=$TFgr.win
	SetVariable $("TuningRegX1_tab"+num2istr(kTuningTab)) pos={300,90},size={150,20},bodywidth=60,limits={-inf,inf,1},proc=TFSetVariable,title="New X:",value=$(TFgr.SettingsFolder+":TuningRegX1"),win=$TFgr.win
//Set to correct tab
	struct WMTabControlAction TCs
	controlInfo/W=$TFgr.win Tabs
	TCs.tab=V_value
	TCs.win=TFgr.win
	TCs.eventcode=2
	TFTabs(TCs)
end

Function TFCheckBox(CBS) : CheckBoxControl
	struct WMCheckboxAction &CBS
	
	if(CBS.eventCode==2)		
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,CBS.win)
		
		GraphNormal/W=$TFgr.win	//Exits drawing mode if user is editing tuning ties
		TF_UpdateTuningTies(TFgr)

		string ctrlName=stringfromlist(0,CBS.ctrlName,"_")
		StrSwitch(CtrlName)
			case "LiveInterp":
				break
			case "RemoveFliers":
			case "RemoveOutliers":
				if(TFgr.RemoveFliers || TFgr.RemoveOutliers)
					SetVariable $("RemoveSD_tab"+num2istr(kDataSelectionTab)) bodywidth=60,size={150,20},pos={420,45},limits={0,10,0.1},proc=TFSetVariable,title="cutoff (sigma):",value=$(TFgr.SettingsFolder+":RemoveSD")
					SetVariable $("RemoveRG_tab"+num2istr(kDataSelectionTab)) bodywidth=60,size={150,20},pos={420,70},limits={0,inf,0.1},proc=TFSetVariable,title="range:",value=$(TFgr.SettingsFolder+":RemoveRG")
				else
					KillControl $("RemoveSD_tab"+num2istr(kDataSelectionTab))
					KillControl $("RemoveRG_tab"+num2istr(kDataSelectionTab))
				endif
				break
			case "ShowTuned":																				//reverts from tuned x-values to untuned -
	//			TF_LoadSettings(TFgr,TFgr.dwPath,TFgr.win)		//Load settings for new checkbox state
				TF_SetParameterDefaults(TFgr,0)					//Load settings for new checkbox state
				break
		endSwitch

		if(TFgr.LiveInterp)						//if live interpolation is selected, reinterpolate
			TF_InitiateInterpolation(TFgr)
			TF_UpdatePlot(TFgr)
		endif
	endif
end

Function TFSetVariable(SVs) : SetVariableControl
	STRUCT WMSetVariableAction &SVs
	
	GraphNormal/W=$SVs.win	//Exits drawing mode if user is editing tuning ties

	if(SVS.eventcode==2 || SVS.eventcode==1)		
		string TabName
				
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,SVS.win)
		
		TF_UpdateTuningTies(TFgr)

		string ctrlName=stringfromlist(0,SVs.ctrlName,"_")
		StrSwitch(CtrlName)
			case "InterpRes":
				TFgr.PeriodMin=SVS.dval*2					//adjust sampling interval in interpolated data
			case "Xmin":
			case "Xmax":
			case "RemoveSD":
			case "RemoveRG":
				if(TFgr.LiveInterp)
					TF_InitiateInterpolation(TFgr)							//reinterpolate if live interpolation is selected
					TF_UpdatePlot(TFgr)
				endif
				break
			case "InterpSV1":
			case "InterpSV2":
			case "InterpSV3":
				TF_initiateInterpolation(TFgr)
				break
			case "TFDecompSV1":
			case "TFDecompSV2":
			case "TFDecompSV3":
				TF_initiateDecomposition(TFgr)
				break
			case "SpectrumSV1":
			case "SpectrumSV2":
			case "SpectrumSV3":
				TF_initiateSpectrum(TFgr)
				break
			case "SigLevel":
				TF_AR1(TFgr)
				break
			case "CTabTau":
				TFgr.CTabMod=exp(-p/TFgr.CTabTau)			//adjusts nonlinearity of color table
			case "CTabMax":
			case "CTabMin":
			case "PTickSpace":
			case "FTickSpace":
			case "LabelSpace":
				break
		EndSwitch
		TF_UpdatePlot(TFgr)
	endif	
End

Function TFWBListbox(SelectedItem, EventCode,win,ControlName)
	string SelectedItem,win,ControlName
	variable EventCode
	
	if(EventCode==WMWS_SelectionChanged)

		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,win)
		
		controlName=stringfromlist(0,controlName,"_")

		DoWindow/K TuningTransferCurve				//kill age/depth plot - unsophisticated way of keeping the age/depth plot from being out of sync with data
		strSwitch(ControlName)
			case "DataWaveSelect":
				if(SVAR_exists(TFgr.dwPath))
					if(stringmatch(selectedItem,TFgr.dwPath))
						break
					endif
				endif
				TF_LoadSettings(TFgr,selectedItem,win)
				break
			case "TimeWaveSelect":
				if(SVAR_exists(TFgr.xwPath))
					if(stringmatch(selectedItem,TFgr.xwPath))
						break
					endif
				endif
				wave TFgr.xw=$selectedItem
				TFgr.xwPath=getWavesDataFolder(TFgr.xw,2)
				TF_MakeRefs(TFgr,TFgr.win)
				TF_SetParameterDefaults(TFgr,1)
				break
		endSwitch
		if(TFgr.LiveInterp)
			TF_InitiateInterpolation(TFgr)
		endif
		TF_UpdatePlot(TFgr)
	endif
end

Function TFPopup(PUs)
	struct WMPopupAction &PUs
	
	if(PUs.eventCode==2)
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,PUs.win)
	
		GraphNormal/W=$PUs.win	//Exits drawing mode if user is editing tuning ties
		TF_UpdateTuningTies(TFgr)
	
		string ctrlName=stringfromlist(0,PUs.ctrlName,"_")
		StrSwitch(ctrlName)
			case "InterpType":								//change in interpolation method
				TFgr.InterpType=PUs.popStr
				TF_InterpOptions(TFgr)
				if(TFgr.LiveInterp)								//reinterpolate if live interpolation is selected
					TF_InitiateInterpolation(TFgr)
				endif
				break
			case "InterpPM1":
				TFgr.InterpPM0=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "InterpPM2":
				TFgr.InterpPM1=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "InterpPM3":
				TFgr.InterpPM2=PUs.popStr
				TF_InitiateInterpolation(TFgr)
				break
			case "DecompType":									//change in time-frequency decomposition
				TFgr.DecompType=PUs.popStr
				TF_MakeRefs(TFgr,TFgr.win)
	//			TFgr.CTabMax=nan;TFgr.CTabMin=nan											//this forces a recalculation of the color table
				TF_DecompOptions(TFgr)
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM1":
				TFgr.TFDecompPM0=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM2":
				TFgr.TFDecompPM1=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "TFDecompPM3":
				TFgr.TFDecompPM2=PUs.popStr
				TF_InitiateDecomposition(TFgr)
				break
			case "SpectrumType":				//change in spectrum decomposition - automatically recalculates spectrum
				TFgr.SpectrumType=PUs.popStr
				TF_MakeRefs(TFgr,TFgr.win)
				TF_SpectrumOptions(TFgr)
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM1":
				TFgr.SpectrumPM0=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM2":
				TFgr.SpectrumPM1=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "SpectrumPM3":
				TFgr.SpectrumPM2=PUs.popStr
				TF_InitiateSpectrum(TFgr)
				break
			case "VertAxis":				//change in (non)linearity of period and frequency axes
				TFgr.VertAxis=PUs.popStr
				break
			case "AmpAxis":			//change from/to log scale for spectrum amplitude
				TFgr.AmpAxis=PUs.popStr
				ModifyGraph log(Amplitude)=whichlistitem(TFgr.AmpAxis,KPopupvalsAmpAxis)
				break
			case "CTabMode":
				TFgr.CTabMode=PUs.popStr
				break
			case "Norms":				//change in normalization of time-frequency amplitudes
				TFgr.Norms=PUs.popStr
				TFgr.CTabMax=nan;TFgr.CTabMin=nan		//force change in color table limits
				break
		EndSwitch
		TF_UpdatePlot(TFgr)
	endif
End

Function TFTabs(TCs)
	struct WMTabControlAction &TCs

	if(TCs.eventcode==2)
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,TCs.win)
		
		GraphNormal/W=$TFgr.win	//Exits drawing mode if user is editing tuning ties
		
		String TabbedControls= ControlNameList("", ";", "*_tab*")					//gather control names
		
		String selectedTab="*_tab"+num2istr(TCs.tab)
		String selectedCtrls=ListMatch(TabbedControls,selectedTab)			//gather names of controls in selected tab
		String unselectedCtrls=ListMatch(TabbedControls,"!"+selectedTab)	//gather names of controls in other tabs
	
		ModifyControlList unselectedCtrls disable=1	// hide
		ModifyControlList selectedCtrls disable=0		// show
	endif
End

Function TFButton(BS) : ButtonControl
	STRUCT WMButtonAction &BS
	
	if(BS.eventcode==2)
		variable i
		string Settings
			
		struct TF_GlobalRefs TFgr
		TF_MakeRefs(TFgr,BS.win)
		
		GraphNormal/W=$BS.win	//Exits drawing mode if user is editing tuning ties
		TF_UpdateTuningTies(TFgr)

		string ctrlName=stringfromlist(0,BS.ctrlName,"_")
		strswitch(ctrlName)
			case "DisplayHelp":		//displays help file at subtopic for current tab
				controlInfo/W=$BS.win Tabs
				DisplayHelpTopic/k=1 "Using the Time Frequency plot["+S_Value+"]"
				break
			case "TuneStep":		//recalculate tuned x-value wave based on current tuning tie picks
				TF_TuneStep(TFgr)
				TFgr.ShowTuned=1
				TF_MakeRefs(TFgr,TFgr.win)
				TF_InitiateInterpolation(TFgr)
				TF_InitiateSpectrum(TFgr)
				TF_InitiateDecomposition(TFgr)
				TF_UpdatePlot(TFgr)
				break
			case "EditPicks":		//switch to drawing mode
				TF_InitiateTuning(TFgr,reset=0)
				break
			case "ResetPicks":		//reset tuning ties and switch to drawing mode
				TF_InitiateTuning(TFgr,reset=1)
				break
			case "Decompose":		//do time-frequency decomposition
				TF_InitiateDecomposition(TFgr)
				TF_UpdatePlot(TFgr)
				break
			case "CopyPlot":			//duplicates the current plot area of the window, so that it can be edited, compared, etc.
				TF_UpdatePlot(TFgr,dup=1)
				break
			case "ResetValues":		//reset parameters to defaults based on data
				TF_SetParameterDefaults(TFgr,0)
				if(TFgr.LiveInterp)
					TF_InitiateInterpolation(TFgr)
				endif
				TF_UpdatePlot(TFgr)
				break
			case "DoInterpolate":	//interpolate
				TF_InitiateInterpolation(TFgr)
				TF_UpdatePlot(TFgr)
				break
			case "DisplayCurve":	//make new graph with x-value vs tuned x-value
				TF_PlotTuningTransfer(TFgr)
				break
			case "SSA":						//eventually, should walk user through SSA detrending of data
				SSAdisplay()
				break
		endSwitch
	endif
End

Function TF_InterpOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct InterpolationParameters IPs
	IPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	IPs.Xmin=TFgr.Xmin;IPs.Xmax=TFgr.Xmax
	IPs.interpRes=TFgr.interpRes
	
	FuncRef INTERPOLATION_prototype InterpFunc=$("INTERPOLATION_"+TFgr.InterpType)
	InterpFunc(IPs)
	string InterpSettingsFolder=TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType
	NewDataFolderPath(InterpSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=InterpSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="InterpSV"+num2istr(i+1)+"_tab"+num2istr(kInterpolationTab)
		if(strlen(IPs.SV.name[i]))
			if(NVAR_exists(gSV))
				IPs.SV.value[i]=gSV
			else
				variable/g $SVpath=IPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={205,20+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.win	
			setvariable $ctrlName limits={IPs.SV.low[i],IPs.SV.high[i],IPs.SV.inc[i]},title=IPs.SV.name[i],value=$SVpath,win=$TFgr.win
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=InterpSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="InterpPM"+num2istr(i+1)+"_tab"+num2istr(kInterpolationTab)
		if(strlen(IPs.PM.name[i]))
			if(SVAR_exists(gPM))
				IPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=IPs.PM.popStr[i]
			endif
			gPMvalue=IPs.PM.value[i]
			PopupMenu $ctrlName,pos={355,20+i*25},proc=TFPopup,title=IPs.PM.name[i],win=$TFgr.win
			PopupMenu $ctrlName value=#(PMpath+"list"),mode=(whichlistitem(IPs.PM.popStr[i],IPs.PM.value[i])+1),win=$TFgr.win
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
end

Function TF_InitiateInterpolation(TFgr)		//interpolate data using user-selected method
	struct TF_GlobalRefs &TFgr
		
	struct InterpolationParameters IPs
	IPs.getParams=0					//setting to calculate spectrum
	FuncRef INTERPOLATION_prototype InterpFunc=$("INTERPOLATION_"+TFgr.InterpType)
	string InterpSettingsFolder=TFgr.SettingsFolder+":Interpolation:"+TFgr.InterpType
	NewDataFolderPath(InterpSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)		//loop to get values of setvariable parameters
		SVpath=InterpSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			IPs.SV.value[i]=gSV
		else
			IPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)		//loop to get values of popup menu parameters
		PMpath=InterpSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			IPs.PM.popStr[i]=gPM
			IPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			IPs.PM.popStr[i]=""
			IPs.PM.popNum[i]=nan
		endif
	endfor
	
//(re)create interpolated data wave
	string IntName=TFgr.TFfolder+":IntData"
	make/o/d $IntName;wave TFgr.dw_int=$IntName

//the following code duplicates the original data and x waves to avoid altering them during pre-processing and interpolation
	string ProcName=TFgr.SettingsFolder+":ProcData"
	string ProcXName=TFgr.SettingsFolder+":ProcX"
	variable np,maxpt,minpt
	duplicate/o TFgr.dw,$ProcName
	wave dw=$ProcName
	if(waveexists(TFgr.Plotxw))			//if there is an x-value wave; references the plotted xw in case the data is being tuned
		duplicate/o TFgr.Plotxw,$ProcXName
		wave xw=$ProcXName
	else
		duplicate/o TFgr.dw,$ProcXName	//creates dummy x-value wave with values set to x-scaling of dw
		wave xw=$ProcXName
		xw=x
	endif
//removes nan in both xw and dw
	sort dw,xw,dw
	wavestats/q/m=0 dw
	redimension/n=(v_npnts) xw,dw
	sort xw,xw,dw					//put in x-value order
	wavestats/q/m=0 xw
	redimension/n=(v_npnts) xw,dw

//truncate data to user-selected range
	minpt=max(0,binarysearch(xw,TFgr.Xmin));maxpt=binarysearch(xw,TFgr.Xmax)		//locate user-selected end-points
	if(maxpt<0)
		maxpt=numpnts(dw)-1
	endif
	redimension/n=(maxpt+1) xw,dw
	deletepoints 0,minpt,xw,dw
	
	TFgr.Xmin=max(TFgr.Xmin,xw[0])//set xmin and xmax to values closest to user selected values
	TFgr.Xmax=min(TFgr.Xmax,xw[numpnts(xw)-1])
	
//set scaling for interpolated data wave
	np=trunc(abs((TFgr.Xmax-TFgr.Xmin)/(TFgr.InterpRes))+1)		//calculate number of points in interpolated wave
	np-=mod(np,2)													//make sure it's a multiple of 2, otherwise FFT fails
	redimension/n=(np) TFgr.dw_int
	setscale/p x TFgr.Xmin,TFgr.InterpRes,TFgr.dw_int
	
//remove fliers and outliers
	if(TFgr.RemoveFliers)						//user wants fliers removed
		RemoveFliers(TFgr.RemoveRG,TFgr.RemoveSD,dw,xw=xw)
	endif
	if(TFgr.RemoveOutliers)						//user wants outliers removed
		RemoveOutliers(TFgr.RemoveRG,TFgr.RemoveSD,dw,xw=xw)
	endif
	
	IPs.Xmin=TFgr.Xmin;IPs.Xmax=TFgr.Xmax
	IPs.interpRes=TFgr.interpRes
	
	wave IPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)
	wave IPs.dw=$getwavesdatafolder(dw,2)
	wave IPs.xw=$getwavesdatafolder(xw,2)
	
	InterpFunc(IPs)		//calls spectrum calculation wrapper function
	
	ReplaceNaN(TFgr.dw_int,0,1)	//interpolated waves will have NaNs where there are not enough data points - this inserts flat intervals (spectrally relatively neutral) for NaNs
end

Function TF_DecompOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct TFDecompParameters TFPs
	TFPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	FuncRef TFDECOMPOSITION_prototype DecompFunc=$("TFDECOMPOSITION_"+TFgr.DecompType)
	DecompFunc(TFPs)
	string DecompSettingsFolder=TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType
	NewDataFolderPath(DecompSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=DecompSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="TFDecompSV"+num2istr(i+1)+"_tab"+num2istr(KTFDecompTab)
		if(strlen(TFPs.SV.name[i]))
			if(NVAR_exists(gSV))
				TFPs.SV.value[i]=gSV
			else
				variable/g $SVpath=TFPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={255,20+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.win	
			setvariable $ctrlName limits={TFPs.SV.low[i],TFPs.SV.high[i],TFPs.SV.inc[i]},title=TFPs.SV.name[i],value=$SVpath,win=$TFgr.win
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=DecompSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="TFDecompPM"+num2istr(i+1)+"_tab"+num2istr(kTFDecompTab)
		if(strlen(TFPs.PM.name[i]))
			if(SVAR_exists(gPM))
				TFPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=TFPs.PM.popStr[i]
			endif
			gPMvalue=TFPs.PM.value[i]
			PopupMenu $ctrlName,pos={505,20+i*25},proc=TFPopup,title=TFPs.PM.name[i],win=$TFgr.win
			PopupMenu $ctrlName value=#(PMpath+"list"),mode=(whichlistitem(TFPs.PM.popStr[i],TFPs.PM.value[i])+1),win=$TFgr.win
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
end

Function TF_InitiateDecomposition(TFgr)	//calculate time-frequency spectrum
	struct TF_GlobalRefs &TFgr
	
	variable MemSize

//	TFgr.CTabMax=nan;TFgr.CTabMin=nan	//forces recalculation of color table
	TF_AR1(TFgr)					//calculate expected AR1 red noise spectrum

	struct TFDecompParameters TFPs
	TFPs.getParams=0		//makes decomposition function do the calculation
	FuncRef TFDECOMPOSITION_prototype DecompFunc=$("TFDECOMPOSITION_"+TFgr.DecompType)

	string DecompSettingsFolder=TFgr.SettingsFolder+":TFDecomposition:"+TFgr.DecompType
	NewDataFolderPath(DecompSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=DecompSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			TFPs.SV.value[i]=gSV
		else
			TFPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=DecompSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			TFPs.PM.popStr[i]=gPM
			TFPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			TFPs.PM.popStr[i]=""
			TFPs.PM.popNum[i]=nan
		endif
	endfor

	TFPs.Xmin=TFgr.Xmin;TFPs.Xmax=TFgr.Xmax
	TFPs.Fmin=1/TFgr.PeriodMax;TFPs.Fmax=1/TFgr.PeriodMin
	if(waveexists(TFgr.xw))
		wave TFPs.dw=$getwavesdatafolder(TFgr.dw,2);wave TFPs.xw=$getwavesdatafolder(TFgr.xw,2)
	endif
	wave TFPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)

//make result waves
	NewDataFolderPath(TFgr.TFDecompFolder)
	make/o $(TFgr.TFDecompFolder+":magnitude"),$(TFgr.TFDecompFolder+":BGnorm"),$(TFgr.TFDecompFolder+":COI")
	make/o $(TFgr.TFDecompFolder+":Frequency"),$(TFgr.TFDecompFolder+":Period"),$(TFgr.TFDecompFolder+":scale")
	TF_MakeRefs(TFgr,TFgr.win)
	wave TFPs.magnitude=$(TFgr.TFDecompFolder+":magnitude"),TFPs.coi=$(TFgr.TFDecompFolder+":COI")
	wave TFPs.frequency=$(TFgr.TFDecompFolder+":Frequency"),TFPs.period=$(TFgr.TFDecompFolder+":Period")
	wave TFPs.scale=$(TFgr.TFDecompFolder+":scale")
	
	DecompFunc(TFPs)

	TF_Sum(TFgr.TFMag)

//background amplitude normalization
	duplicate/o TFgr.TFMag,TFgr.TFBGnorm
	TFgr.TFBGnorm=TFgr.TFMag[p][q]/TFgr.AR1Mag(y)
end

Function TF_SpectrumOptions(TFgr)	//adds controls specfic to spectral estimate methods
	struct TF_GlobalRefs &TFgr
	
	struct SpectrumParameters SPs
	SPs.getParams=1		//makes spectrum function return information needed to build parameter controls
	FuncRef SPECTRUM_prototype SpectrumFunc=$("SPECTRUM_"+TFgr.SpectrumType)
	SpectrumFunc(SPs)
	string SpecSettingsFolder=TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType
	NewDataFolderPath(SpecSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)					//loop to build setvariable controls for spectrum parameters
		SVpath=SpecSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		ctrlName="SpectrumSV"+num2istr(i+1)+"_tab"+num2istr(kSpectrumTab)
		if(strlen(SPs.SV.name[i]))
			if(NVAR_exists(gSV))
				SPs.SV.value[i]=gSV
			else
				variable/g $SVpath=SPs.SV.value[i]
			endif
			SetVariable $ctrlName pos={255,20+i*25},size={150,20},bodywidth=70,proc=TFSetVariable,win=$TFgr.win	
			setvariable $ctrlName limits={SPs.SV.low[i],SPs.SV.high[i],SPs.SV.inc[i]},title=SPs.SV.name[i],value=$SVpath,win=$TFgr.win
		else
			killvariables/z gSV
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
	for(i=0;i<3;i+=1)				//loop to build popup menu controls for spectrum parameters
		PMpath=SpecSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		string/g $(PMpath+"list")
		SVAR gPMvalue=$(PMpath+"list")
		ctrlName="SpectrumPM"+num2istr(i+1)+"_tab"+num2istr(kSpectrumTab)
		if(strlen(SPs.PM.name[i]))
			if(SVAR_exists(gPM))
				SPs.PM.popStr[i]=gPM
			else
				string/g $PMpath=SPs.PM.popStr[i]
			endif
			gPMvalue=SPs.PM.value[i]
			PopupMenu $ctrlName,pos={505,20+i*25},proc=TFPopup,title=SPs.PM.name[i],win=$TFgr.win
			PopupMenu $ctrlName value=#(PMpath+"list"),mode=(whichlistitem(SPs.PM.popStr[i],SPs.PM.value[i])+1),win=$TFgr.win
		else
			killstrings/z gPM,gPMvalue
			killcontrol/W=$TFgr.win $ctrlName
		endif
	endfor
end

Function TF_InitiateSpectrum(TFgr)		//calculates spectrum
	struct TF_GlobalRefs &TFgr
	
	TF_AR1(TFgr)					//calculates expected AR1 red noise spectrum

	struct SpectrumParameters SPs
	SPs.getParams=0					//setting to calculate spectrum
	FuncRef SPECTRUM_prototype SpectrumFunc=$("SPECTRUM_"+TFgr.SpectrumType)
	string SpecSettingsFolder=TFgr.SettingsFolder+":Spectrum:"+TFgr.SpectrumType
	NewDataFolderPath(SpecSettingsFolder)
	variable i
	string SVpath,PMpath,ctrlName

	for(i=0;i<3;i+=1)		//loop to get values of setvariable parameters
		SVpath=SpecSettingsFolder+":SV"+num2istr(i)
		NVAR/Z gSV=$SVpath
		if(NVAR_exists(gSV))
			SPs.SV.value[i]=gSV
		else
			SPs.SV.value[i]=nan
		endif
	endfor
	for(i=0;i<3;i+=1)		//loop to get values of popup menu parameters
		PMpath=SpecSettingsFolder+":PM"+num2istr(i)
		SVAR/Z gPM=$PMpath
		SVAR/Z gPMvalue=$(PMpath+"list")
		if(SVAR_exists(gPM))
			SPs.PM.popStr[i]=gPM
			SPs.PM.popNum[i]=whichlistitem(gPM,gPMvalue)+1
		else
			SPs.PM.popStr[i]=""
			SPs.PM.popNum[i]=nan
		endif
	endfor
	
	SPs.Xmin=TFgr.Xmin;SPs.Xmax=TFgr.Xmax
	SPs.Fmin=1/TFgr.PeriodMax;SPs.Fmax=1/TFgr.PeriodMin
	if(waveexists(TFgr.xw))
		wave SPs.dw=$getwavesdatafolder(TFgr.dw,2);wave SPs.xw=$getwavesdatafolder(TFgr.xw,2)
	endif
	wave SPs.dw_int=$getwavesdatafolder(TFgr.dw_int,2)
	NewDataFolderPath(TFgr.SpectrumFolder)											//Create spectrum subfolder, if it doesn't exist
	make/o $(TFgr.SpectrumFolder+":Amplitude")
	wave TFgr.PgramMag=$(TFgr.SpectrumFolder+":Amplitude"),SPs.magnitude=$(TFgr.SpectrumFolder+":Amplitude")
	
	SpectrumFunc(SPs)		//calls spectrum calculation wrapper function
	
	duplicate/o TFgr.PgramMag,$(TFgr.SpectrumFolder+":Frequency"),$(TFgr.SpectrumFolder+":Period")
	wave TFgr.PgramFrequency=$(TFgr.SpectrumFolder+":Frequency"),TFgr.PgramPeriod=$(TFgr.SpectrumFolder+":Period")
	TFgr.PgramFrequency=x;TFgr.PgramPeriod=1/x
end

Function TF_InitiateTuning(TFgr[,reset])	//switches to drawing mode to edit tuning ties
	struct TF_GlobalRefs &TFgr
	variable reset
		
	
	newdatafolderPath(TFgr.TFfolder+":Tuning")		//create data folder for tuning ties
	if(reset)// || !TFgr.ShowTuned)							//create tuning tie waves if never tuned or user selected reset
		make/o/n=2 $(TFgr.TFfolder+":Tuning:frequencyTie"),$(TFgr.TFfolder+":Tuning:periodTie")
		if(waveexists(TFgr.xw))
			make/o/n=2 $(TFgr.xwPath+"_origTie"),$(TFgr.xwPath+"_tunedTie")
		else
			make/o/n=2 $(TFgr.dwPath+"X_origTie"),$(TFgr.dwPath+"X_tunedTie")
		endif
		TF_MakeRefs(TFgr,TFgr.win)
		TFgr.tunedTie={TFgr.Xmin,TFgr.Xmax}
		TFgr.periodTie=(TFgr.PeriodMax+TFgr.PeriodMin)/2
		TFgr.frequencyTie=1/TFgr.periodTie
		TF_updatePlot(TFgr)				//after creation of waves, update plot to show them
	endif
	Strswitch(TFgr.VertAxis)
		case "Linear Frequency":
			GraphWaveEdit/M frequencyTie
			break
		case "Linear Period":
		case "Log(2)":
		case "Log(10)":
			GraphWaveEdit/M periodTie
			break
	endswitch
end

Function TF_UpdateTuningTies(TFgr)		//Function for dependency that keeps tuning ties consistent with frequency plotting method
	struct TF_GlobalRefs &TFgr

	if(!(waveexists(TFgr.origTie) && waveexists(TFgr.tunedTie) && waveexists(TFgr.frequencyTie) && waveexists(TFgr.periodTie)))
		return 0
	endif
	redimension/n=(numpnts(TFgr.PlotXTie)) TFgr.origTie,TFgr.tunedTie,TFgr.frequencyTie,TFgr.periodTie
	
	Strswitch(TFgr.VertAxis)
		case "Linear Frequency":
			TFgr.periodTie=1/TFgr.frequencyTie
			break
		case "Linear Period":
		case "Log(2)":
		case "Log(10)":
			TFgr.frequencyTie=1/TFgr.periodTie
			break
	endswitch
end

Function TF_AR1(TFgr)			//calculates expected AR1 red noise spectrum
	struct TF_GlobalRefs &TFgr
	
	struct AR1_Params AR1p
	AR1_getParamsSimple(TFgr.dw_int,AR1p)
	
	variable N=numpnts(TFgr.dw_int)
	newdatafolderPath(TFgr.AR1folder)
	make/o/n=((0.5/TFgr.interpRes)/(1/(TFgr.XMax-TFgr.XMin))) $(TFgr.AR1folder+":Amplitude"),$(TFgr.AR1folder+":period"),$(TFgr.AR1folder+":frequency")
	wave TFgr.AR1Mag=$(TFgr.AR1folder+":Amplitude"),TFgr.AR1period=$(TFgr.AR1folder+":period"),TFgr.AR1frequency=$(TFgr.AR1folder+":frequency")
	setscale/p x 0,1/(TFgr.XMax-TFgr.XMin),TFgr.AR1Mag,TFgr.AR1period,TFgr.AR1frequency
	TFgr.AR1Mag=AR1p.tau/TFgr.interpRes*AR1p.alpha*sqrt(1/(1+(2*pi*x*AR1p.tau)^2)/N)*pi/2	//ref?
	TFgr.AR1Mag*=sqrt(statsInvChiCDF(TFgr.sigLevel/100,2))												//spectrum should be chi2 distributed
	TFgr.AR1frequency=x;TFgr.AR1period=1/x
end

Function TF_Sum(TFmat)					//calculates spectrum by taking mean of time-frequency decomposition
	wave TFmat
	
	variable i,npts=dimsize(TFmat,0),ncols=dimsize(TFmat,1)
	string name=GetWavesDataFolder(TFmat,1)+"GlobalSpec"
	make/o/n=(ncols) $(name)
	wave GlobalSpec=$(name)
	setscale/p x DimOffset(TFmat,1),DimDelta(TFmat,1),GlobalSpec
	GlobalSpec=0
	for(i=0;i<ncols;i+=1)
		ImageStats/G={0,npts-1,i,i} TFmat
		GlobalSpec[i]=V_rms
	endfor
end

function TF_UpdatePlot(TFgr[,dup])
	struct TF_GlobalRefs &TFgr
	variable dup		//if >0, this creates a duplicate of the current plot rather than updating
		
	TF_MakeRefs(TFgr,TFgr.win)
	variable numticks
	
	if(dup)				//if duplicating rather than updating
		display/W=(5,164,705,764)/N=$(nameofwave(TFgr.dw)+"_TFplot")
		Modifygraph margin(left)=72,margin(bottom)=72,margin(right)=(72)
	else
//Remove old plots:
		ClearGraph()
		ColorScale/N=AmpScale/K
	endif
		
	if(waveexists(TFgr.dw))											//if data wave has been selected, then update the plot
		if(WaveExists(TFgr.Plotxw))	//plot dw vs xw, if one has been selected
			appendToGraph/c=(0,0,0)/l=Data/b=Position TFgr.dw vs TFgr.Plotxw
		else
			appendToGraph/c=(0,0,0)/l=Data/b=Position TFgr.dw
		endif
		if(WaveExists(TFgr.dw_int) && (numpnts(TFgr.dw_int)>0))	//plot interpolated data, if it exists
			AppendToGraph/l=Data/b=Position TFgr.dw_int
			wavestats/q/M=1 TFgr.dw_int								//autoscales to interpolated data - should be controllable by user
			setaxis/A=2 Data
		endif
		setaxis Position TFgr.Xmin,TFgr.Xmax

		StrSwitch(TFgr.SpectrumType)					//plot spectrum
			case "Periodogram":			//user has selected Periodogram
				if(waveexists(TFgr.PgramMag))
					AppendToGraph/c=(0,0,0)/B=Amplitude/L=Period/VERT TFgr.PgramMag vs TFgr.PlotXPgram
				endif
				break
			case "Lomb":			//user has selected Periodogram
				if(waveexists(TFgr.PgramMag))
					AppendToGraph/c=(0,0,0)/B=Amplitude/L=Period/VERT TFgr.PgramMag vs TFgr.PlotXPgram
				endif
				break
			case "BlackmanTukey":			//user has selected Periodogram
				if(waveexists(TFgr.PgramMag))
					AppendToGraph/c=(0,0,0)/B=Amplitude/L=Period/VERT TFgr.PgramMag vs TFgr.PlotXPgram
				endif
				break
		EndSwitch
		if(waveexists(TFgr.PlotTFdecomp))
			AppendImage/l=Period/b=Position TFgr.PlotTFdecomp vs {*,TFgr.PlotXTFdecomp}
			AppendToGraph/C=(0,0,65535)/B=Amplitude/L=Period/VERT TFgr.TFGlobalSpec vs TFgr.PlotXTFdecomp
			if(waveexists(TFgr.PlotXTie) && waveexists(TFgr.origTie) && waveexists(TFgr.tunedTie))
				appendtograph/b=Position/l=Period TFgr.PlotXTie vs TFgr.tunedTie
				modifygraph mode($nameofwave(TFgr.PlotXTie))=4
			endif
			if(!WaveExists(TFgr.CTabMod))
				make/o/n=100 $(TFgr.TFFolder+":CTabMod")
				wave TFgr.CTabMod=$(TFgr.TFFolder+":CTabMod")
			endif
			TFgr.CTabMod=exp(-p/TFgr.CTabTau)
			if(numtype(TFgr.CTabMin))
				wavestats/q/m=1 TFgr.PlotTFdecomp;TFgr.CTabMin=(v_min+v_avg)/10
			endif
			if(numtype(TFgr.CTabMax))
				wavestats/q/m=1 TFgr.PlotTFdecomp;TFgr.CTabMax=(v_max)
			endif
			ModifyImage $nameofwave(TFgr.PlotTFdecomp) ctab= {TFgr.CTabMin,TFgr.CTabMax,$TFgr.CTabMode,0}
			ModifyImage $nameofwave(TFgr.PlotTFdecomp) lookup=TFgr.CTabMod
			ColorScale/C/N=AmpScale/F=0/A=MB/E=0/X=10/Y=-20 vert=0,widthPct=50,height=10,log=1,minor=1
			ColorScale/C/N=AmpScale image=$nameofwave(TFgr.PlotTFdecomp)
		endif
		if(waveexists(TFgr.AR1Mag))			//if AR1 expected red noise spectrum has been calculated
			AppendToGraph/C=(65535,0,0)/B=Amplitude/L=Period/VERT TFgr.AR1Mag vs TFgr.PlotXAR1
		endif

		Modifygraph/Z axisenab(Data)={0.8,1},axisenab(Position)={0,0.8}
		if(!waveexists(TFgr.vertP))
			make/o/t $(TFgr.TFfolder+":vertP"),$(TFgr.TFfolder+":vertF")
			make/o $(TFgr.TFfolder+":vertPy"),$(TFgr.TFfolder+":vertFy")
			wave/t TFgr.vertP=$(TFgr.TFfolder+":vertP"),TFgr.vertF=$(TFgr.TFfolder+":vertF")
			wave TFgr.vertPy=$(TFgr.TFfolder+":vertPy"),TFgr.vertFy=$(TFgr.TFfolder+":vertFy")
		endif
		NewFreeAxis/o/r Frequency
		if(whichListItem("Period",AxisList(TFgr.win))>0)
			strswitch(TFgr.VertAxis)						//differentiate between linear and log axes
				case "Linear frequency":
					setaxis/R Period 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					setaxis/R Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph log(Period)=0,log(Frequency)=0
					numticks=ceil((1/TFgr.PeriodMin)/TFgr.FTickSpace)
					redimension/n=(numticks) TFgr.vertF,TFgr.vertFy,TFgr.vertP,TFgr.vertPy
					TFgr.VertFy=TFgr.FTickSpace*p;TFgr.VertF=num2str(TFgr.VertFy)
					TFgr.VertPy=1/SigDigits(1/TFgr.vertFy,2);TFgr.VertP=num2str(1/TFgr.VertPy)
					ModifyGraph userticks(Period)={TFgr.vertPy,TFgr.vertP}
//					ModifyGraph userticks(Frequency)={TFgr.vertFy,TFgr.vertF}
//					TFgr.vertP[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertP[p],"")
					TFgr.vertF[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertF[p],"")
					break
				case "Linear Period":
					setaxis Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis Frequency TFgr.PeriodMin,TFgr.PeriodMax
					ModifyGraph log(Period)=0,log(Frequency)=0
					numticks=ceil(TFgr.PeriodMax/TFgr.PTickSpace)
					redimension/n=(numticks) TFgr.vertP,TFgr.vertPy,TFgr.vertF,TFgr.vertFy
					TFgr.VertPy=TFgr.PTickSpace*p;TFgr.VertP=num2str(TFgr.VertPy)
					TFgr.VertFy=1/SigDigits(1/TFgr.VertPy,2);TFgr.VertF=num2str(1/Tfgr.VertFy)
//					ModifyGraph userticks(Period)={TFgr.vertPy,TFgr.vertP}
					ModifyGraph userticks(Frequency)={TFgr.vertFy,TFgr.vertF}
					TFgr.vertP[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertP[p],"")
//					TFgr.vertF[]=selectstring(mod((p),TFgr.LabelSpace),TFgr.vertF[p],"")
					break
				case "Log(10)":
					setaxis Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph log(Period)=1,log(Frequency)=1
					break
				case "Log(2)":
					setaxis Period TFgr.PeriodMin,TFgr.PeriodMax
					setaxis Frequency 1/TFgr.PeriodMin,1/TFgr.PeriodMax
					ModifyGraph log(Period)=2,log(Frequency)=2
					break
			endSwitch
			Modifygraph/Z axisenab(Period)={0,0.75},axisenab(Frequency)={0,0.75}
		endif
		if(whichListItem("Amplitude",AxisList(TFgr.win))>0)
			ModifyGraph/Z log(Amplitude)=whichlistitem(TFgr.AmpAxis,KPopupvalsAmpAxis)
			Modifygraph axisenab(Amplitude)={0.85,1}
			wavestats/q/m=(1)/z/R=(1/TFgr.PeriodMax,1/TFgr.PeriodMin) TFgr.PgramMag
			setaxis/a=2 Amplitude
		endif
		ModifyGraph lblPos=60
		Modifygraph freepos=0,minor=1
		doupdate
	//	TitleBox Instructions pos={100,0},frame=1,fSize=12,fColor=(65535,0,0),size={475,100},variable=TitleStr
	//	killstrings TitleStr
	endif
end

function SigDigits(val,dig)			//utility that returns a value truncated to dig number of digits
	variable val,dig
	
	variable i,ret
	for(i=1E-20;i<=1E20;i*=10)
		ret=trunc(val*i)
		if(ret)
			i*=10^(dig-1)
			ret=round(val*i)/i
			break
		endif
	endfor
	return ret
end

Function TF_TuneStep(TFgr)					//calculates new tuned x-value waves based on user-selected tie points
	struct TF_GlobalRefs &TFgr
	
	string tunedName
	
	variable nTies=numpnts(TFgr.tunedTie),RegCalc,i
		
//Add end nodes
	if(TFgr.tunedTie[0]<=TFgr.XMin)			//ties placed prior to or at start of series
		do										//check for multiple ties out of range - delete extras
			if(TFgr.tunedTie[1]<=TFgr.XMin)
				deletepoints 0,1,TFgr.tunedTie,TFgr.origTie,TFgr.frequencyTie,TFgr.periodTie
				nTies-=1
			else
				break
			endif
		while(1)
		TFgr.tunedTie[0]=TFgr.XMin			//move to start of series, keeps user-defined frequency
	else											//no tie prior to start of series, so ...
		insertpoints 0,1,TFgr.tunedTie,TFgr.origTie,TFgr.frequencyTie,TFgr.periodTie
		nTies+=1
		TFgr.tunedTie[0]=TFgr.XMin			//add a tie, setting it to same frequency as nearest user-defined tie
		TFgr.frequencyTie[0]=TFgr.frequencyTie[1];TFgr.periodTie[0]=TFgr.periodTie[1]
	endif
	if(TFgr.tunedTie[nTies-1]>=TFgr.XMax)		//ties placed after or at end of series
		do										//check for multiple ties out of range - delete extras
			if(TFgr.tunedTie[nTies-2]>=TFgr.XMax)
				redimension/n=(nTies-1) TFgr.tunedTie,TFgr.origTie,TFgr.frequencyTie,TFgr.periodTie
				nTies-=1
			else
				break
			endif
		while(1)
		TFgr.tunedTie[nTies-1]=TFgr.XMax		//move to end of series, keeps user-defined frequency
	else
		redimension/n=(nTies+1) TFgr.tunedTie,TFgr.origTie,TFgr.frequencyTie,TFgr.periodTie
		nTies+=1
		TFgr.tunedTie[nTies-1]=TFgr.XMax		//add a tie, setting it to same frequency as nearest user-defined tie
		TFgr.frequencyTie[nTies-1]=TFgr.frequencyTie[nTies-2];TFgr.periodTie[nTies-1]=TFgr.periodTie[nTies-2]
	endif
	if(TFgr.ShowTuned)								//if this is a refinement of previous tuning
		TFgr.origTie=round(binarysearchinterp(TFgr.IntX_tuned,TFgr.tunedTie))		//converts to point index in interpolated series
		TFgr.origTie[0]=0;TFgr.origTie[nTies-1]=numpnts(TFgr.IntX_tuned)-1	
		TFgr.frequencyTie[0]*=2*(TFgr.IntX_tuned[TFgr.origTie[p]+0.5]-TFgr.IntX_tuned[TFgr.origTie[p]])/(deltax(TFgr.IntX_tuned))	//converts to frequency relative to original ordinates
		TFgr.frequencyTie[1,nTies-2]*=(TFgr.IntX_tuned[TFgr.origTie[p]+0.5]-TFgr.IntX_tuned[TFgr.origTie[p]-0.5])/(deltax(TFgr.IntX_tuned))	//converts to frequency relative to original ordinates
		TFgr.frequencyTie[nTies-1]*=2*(TFgr.IntX_tuned[TFgr.origTie[p]]-TFgr.IntX_tuned[TFgr.origTie[p]-0.5])/(deltax(TFgr.IntX_tuned))
		TFgr.origTie=leftx(TFgr.IntX_tuned)+deltax(TFgr.IntX_tuned)*TFgr.origTie	//converts to original ordinates
		duplicate/o TFgr.IntX_tuned,tmpTuned
		tmpTuned=x
//		TFgr.IntX_tuned[]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p+0.5])*TFcs.TuningPeriod		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[0]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p],tmptuned[p+0.5])*TFgr.TuningPeriod*2		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[1,numpnts(TFgr.IntX_tuned)-2]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p+0.5])*TFgr.TuningPeriod		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[numpnts(TFgr.IntX_tuned)-1]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p])*TFgr.TuningPeriod*2		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[1,]+=TFgr.IntX_tuned[p-1]		//summed intervals yields new ordinate curve

//***the following code would instead calculate new x series relative to preceding iteration rather than original ordinate
//		TFgr.IntX_tuned[1,]-=tmptuned[p-1]		//calculates ordinate intervals
//		TFgr.IntX_tuned*=faveragexy(TFgr.tunedTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p+0.5])*TFcs.TuningPeriod
//		TFgr.IntX_tuned[1,]+=TFgr.IntX_tuned[p-1]
	else											//if this is first tuning pass
		if(waveexists(TFgr.xw))									//make wave to plot original series against tuned ordinates
			tunedName=getwavesdatafolder(TFgr.xw,2)+"_tuned"
			duplicate/o TFgr.xw,$tunedName
		else
			tunedName=getwavesdatafolder(TFgr.dw,2)+"X_tuned"
			duplicate/o TFgr.dw,$tunedName
		endif
		wave TFgr.xw_tuned=$tunedName
		TFgr.origTie=TFgr.tunedTie				//keep track of original ordinate equivalent to tuning tie ordinate
		duplicate/o TFgr.dw_int,$(TFgr.TFfolder+":IntDataX_tuned"),tmptuned	//tuned ordinate for interpolated data series
		wave TFgr.IntX_tuned=$(TFgr.TFfolder+":IntDataX_tuned")
		tmpTuned=x
//		TFgr.IntX_tuned[]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p+0.5])*TFgr.TuningPeriod		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[1,numpnts(TFgr.IntX_tuned)-2]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p+0.5])*TFgr.TuningPeriod		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[0]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p],tmptuned[p+0.5])*TFgr.TuningPeriod*2		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[numpnts(TFgr.IntX_tuned)-1]=areaXY(TFgr.origTie,TFgr.frequencyTie,tmptuned[p-0.5],tmptuned[p])*TFgr.TuningPeriod*2		//converts frequency to tuned ordinate interval
		TFgr.IntX_tuned[1,]+=TFgr.IntX_tuned[p-1]		//summed intervals yields new ordinate curve
	endif
	if((numtype(TFgr.TuningRegX)==0) && (numtype(TFgr.TuningRegX1)==0))
		RegCalc=TFgr.TuningRegX1-TFgr.IntX_tuned(TFgr.TuningRegX)
		TFgr.IntX_tuned+=RegCalc
	endif
	if(waveexists(TFgr.xw))
		TFgr.xw_tuned=(numtype(TFgr.xw)>0 || TFgr.xw[p]>rightx(TFgr.IntX_tuned)-deltax(TFgr.IntX_tuned) || TFgr.xw[p]<leftx(TFgr.IntX_tuned)) ? nan : TFgr.IntX_tuned(TFgr.xw[p])
	else
		TFgr.xw_tuned=(pnt2x(TFgr.dw,p)>rightx(TFgr.IntX_tuned)-deltax(TFgr.IntX_tuned) || pnt2x(TFgr.dw,p)<leftx(TFgr.IntX_tuned)) ? nan : TFgr.IntX_tuned(pnt2x(TFgr.dw,p))
	endif

//Adjust analysis parameters
	wavestats/q/M=1 TFgr.xw_tuned
	variable fac=(TFgr.XMax-TFgr.XMin)/(v_max-v_min)
	TFgr.InterpRes/=fac
	TFgr.PeriodMax/=fac;TFgr.PeriodMin/=fac
	TFgr.XMax=v_max;TFgr.XMin=v_min

//	TFgr.tunedTie=x2pnt(TFgr.IntX_tuned,TFgr.origTie)
	TFgr.tunedTie=TFgr.IntX_tuned(TFgr.origTie)
	TFgr.frequencyTie=1/TFgr.TuningPeriod;TFgr.periodTie=TFgr.TuningPeriod
	
	killwaves tmpTuned
end

Function TF_PlotTuningTransfer(TFgr)		//displays graph of tuned versus original ordinates (Age vs Depth plot)
	struct TF_GlobalRefs &TFgr
	
	AgeModelPlot(PlotName=TFgr.win+"_AMPlot")
	AppendToGraph TFgr.tunedTie vs TFgr.origTie
	if(waveexists(TFgr.xw))
		AppendToGraph TFgr.xw_tuned vs TFgr.xw
	else
		AppendToGraph TFgr.xw_tuned
	endif
	ModifyGraph mode($nameofwave(TFgr.tunedTie))=3
	label bottom "Original units"
	label left "Tuned units"
end

static Function NewDataFolderPath(path[,set])
	string path
	variable set
	
	variable depth=itemsinlist(path,":"),i
	string partial="root"
	for(i=1;i<depth;i+=1)
		partial+=":"+possiblyquotename(StringFromList(i,path,":"))
		newdatafolder/o $partial
	endfor
	if(set)
		SetDataFolder $partial
	endif
end

static Function ClearGraph([win])
	string win
	
	if(paramisdefault(win))
		win=winname(0,1)
	endif
	dowindow $win
	if(v_flag==0)
		win=winname(0,1)
	endif
	
	string PlotList
	variable i,nwaves
	PlotList=TraceNameList(win,";",1);nwaves=ItemsInList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		RemoveFromGraph/W=$win/Z $stringfromlist(i,PlotList)
	endfor
	PlotList=ImageNameList(win,";");nwaves=ItemsInList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		RemoveImage/W=$win/Z $stringfromlist(i,PlotList)
	endfor
	PlotList=AxisList(win);nwaves=itemsinList(PlotList)
	for(i=nwaves-1;i>=0;i-=1)
		KillFreeAxis/W=$win $stringfromlist(i,PlotList)
	endfor
end

static Function RemoveFliers(rg,sd,lw[,xw])
	wave lw,xw;variable rg,sd
	
	variable np,i,nr=0,no=0,pmin,pmax
	
	np=numpnts(lw)
	duplicate/o lw,tempderiv
	rg/=2
	if(waveexists(xw))
		differentiate tempderiv /X=xw
	else
		differentiate tempderiv
		rg/=deltax(lw)
	endif
//	do
		for(i=0,nr=no;i<np;i+=1)
			if(waveexists(xw))
				pmin=binarysearch(xw,xw[i]-rg)+1;pmax=binarysearch(xw,xw[i]+rg)
				if(pmax==-2)
					pmax=np-1
				endif
				wavestats/q/r=[pmin,pmax] tempderiv
			else
				wavestats/q/r=[max(i-rg,0),min(i+rg,np-1)] tempderiv
			endif
			if((abs(tempderiv[i])>(sd*V_sdev))*(numtype(tempderiv[i])<2))
				lw[i-1,i+1]=NaN;tempderiv[i]=NaN;no+=1
			endif
		endfor
//	while(nr!=no)
	killwaves/z tempderiv
	return nr
end

static Function RemoveOutliers(rg,sd,lw[,xw])
	wave lw,xw;variable rg,sd
	
	variable np,i,no=0,nr=0,pmin,pmax
	
	np=numpnts(lw)
	if(waveexists(xw))
	else
		rg/=deltax(lw)
	endif
//	do
		for(i=0,nr=no;i<np;i+=1)
			if(waveexists(xw))
				pmin=binarysearch(xw,xw[i]-rg)+1;pmax=binarysearch(xw,xw[i]+rg)
				if(pmax==-2)
					pmax=np-1
				endif
				wavestats/q/r=[pmin,pmax] lw
			else
				wavestats/q/r=[max(i-rg,0),min(i+rg,np-1)] lw
			endif
			if((lw[i]>(V_avg+sd*V_sdev)) + (lw[i]<(V_avg-sd*V_sdev)))
				lw[i]=NaN;no+=1
			endif
		endfor
//	while(nr!=no)
	return nr
end


static Function ReplaceNaN(w,at,rg)
	wave w
	variable at	//method of filling: 
				//		0 to fill with the average of the closest values
				//		1 to linearly interpolate between closest values
	variable rg	//number of points to use in defining closest values
	
	Variable i,j,avg,np=numpnts(w)
	
	i=0
	do
		wavestats/q/r=[i,min(i+99,np-1)] w
		if(V_numNaNs>0)
			i-=1
			do
				i+=1
			while((numtype(w[i])==0) & (i<np))
			j=0
			do
				j+=1
			while((numtype(w[i+j])!=0) & (i+j<np))
			if(i==0)
				w[i,i+j-1]=w[i+j]
			elseif(i+j==np)
				w[i,i+j-1]=w[i-1]
			elseif(at)
				wavestats/q/r=[max(i-rg,0),max(i-1,0)] w;avg=V_avg
				wavestats/q/r=[min(i+j,np-1),min(i+j+rg-1,np-1)] w
				w[i,i+j-1]=avg-((avg-V_avg)/j)*(p-i+1)
			else
				wavestats/q/r=[max(i-rg,0),min(i+j+rg-1,np-1)] w
				w[i,i+j-1]=v_avg
			endif
			i+=j
		else
			i+=100
		endif
	while(i<np)
end

